c =======================================================
c subs8: various subroutines used by long8; 
c copyright Milani and Knezevic, 1992, 1997 
c ********************************************************************
c   convun
c   conversion of units for longit/secres
c *********************************************************************
      subroutine convun(pm,pa,pg,ps,pn,aunit,tunit,pmunit,
     +   nplin,nplou,fnl,nnl)
      implicit double precision (a-h,o-z)
c  secular theory frequencies
      dimension pg(8),ps(8),fnl(nnl)
c  masses, semimajor axes and mean motions of the planets
      dimension pm(8),pn(8),pa(8)
c   Gauss k constant
      gk=0.01720209895d0
      pnj=gk*3.6525d2*dsqrt((1.d0+pm(5))/pa(5)**3)
c   unit of time=period of jupiter/dpig
      tunit=1.d0/pnj
c     write(*,*)' tunit=',tunit
c  conversion of frequencies to new time unit
      do 13 j=nplin,nplou
        pg(j)=pg(j)*tunit
 13     ps(j)=ps(j)*tunit
      do 15 n=1,nnl
 15     fnl(n)=fnl(n)*tunit
c   unit mass=mass of sun+jupiter
      pmj=pm(5)
      pmunit=1.d0+pmj
      do 11 j=nplin,nplou
 11     pm(j)=pm(j)/pmunit
      pmsun=1.d0/pmunit
c   unit of length=semimajor axis of jupiter
      aunit=pa(5)
      do 12 j=nplin,nplou
 12     pa(j)=pa(j)/aunit
c  mean motions of planets - n'(jup)=1.d0
      do 14 j=nplin,nplou
 14      pn(j)=dsqrt((pmsun+pm(j))/(pa(j))**3.d0)
      if(abs(pn(5)-1.d0).gt.1.d-15)write(*,*)'pn(5)',pn(5)
      pn(5)=1.d0
      return
      end
c
c **********************************************************************
c  inplda
c  inner planets data (used to compute g0, s0 in longit/secres)
c **********************************************************************
      subroutine inplda(pa,pmv)
      implicit double precision (a-h,o-z)
      dimension pa(8),pmv(8)
c  semimajor axis from Bretagnon (1974)
      pa(1)=0.387099d0
      pa(2)=0.723332d0
      pa(3)=1.d0
      pa(4)=1.523691d0
c  masses from Standish and Campbell (1984), see Nobili et al.(1989)
      pmv(1)=1.d0/6.0236d6
      pmv(2)=1.d0/4.085235d5
      pmv(3)=1.d0/3.289055d5
      pmv(4)=1.d0/3.09871d6
      return
      end
c ****************************************************************
c  gsfour
c subroutine which calculates the degree four corrections of
c asteroid free oscillation frequencies g0,s0. output delta g,s.
c ****************************************************************
      subroutine gsfour(pm,ya,yb,xi,et,pni,pmi,
     *    ani,ami,cg,cs,nplin,nplou)
      implicit real*8(a-h,o-z)
      dimension ya(4),yb(27),xi(8),et(8),pni(8),pmi(8)
      sumxi=0.d0
      sumet=0.d0
      xini=0.d0
      etmi=0.d0
      sumni=0.d0
      summi=0.d0
      do 1 k=nplin,nplou
        sumxi=sumxi+xi(k)**2
        sumet=sumet+et(k)**2
        xini=xini+xi(k)*pni(k)
        etmi=etmi+et(k)*pmi(k)
        sumni=sumni+pni(k)**2
        summi=summi+pmi(k)**2
 1    continue
      cg=pm*((yb(1)-ya(1)/4.d0)*(2.d0*ani+4.d0*sumxi)
     $+(yb(5)+ya(2)/2.d0)*(ami+sumet)+
     $(yb(10)+ya(3)/4.d0)*etmi+(yb(12)-ya(4)/8.d0)*2.d0*xini+
     $yb(2)*sumni+yb(6)*summi)
      cs=pm*((yb(5)+ya(2)/2.d0)*(ani+sumxi)
     $+(yb(3)-ya(2)/4.d0)*(2.d0*ami+4.d0*sumet)+
     $(yb(8)-ya(3)/8.d0)*2.d0*etmi+yb(4)*summi
     $+yb(7)*sumni+yb(14)*xini)
      return
      end
c *********************************************************************
c   g0s0
c   second order/degree frequencies for perihelion and node
c   of the asteroid; planets from nplin to nplou are
c   included in the computation
c   second order effects are included in yd; however, if
c   for a given planet the second order effect is not required,
c   yd is set to zero in lly
c *********************************************************************
      subroutine g0s0(al,ya,pm,g0,s0,nplin,nplou)
      implicit double precision (a-h,o-z)
c  masses of the planets
      dimension pm(8)
c  yuasa coefficients--warning!! firt index is yuasa index,
c    second index is planet index
c  yuasa coefficients  a
      dimension ya(4,8)
      dsl=2.d0/al
      g0=0.d0
      s0=0.d0
      do 45 j=nplin,nplou
        g0=g0+dsl*pm(j)*ya(1,j)
 45     s0=s0+dsl*pm(j)*ya(2,j)
      return
      end
c ********************************************************************
c  forced
c  forced oscillation amplitudes.
c  second order effects are accounted for; if they are not
c  required, yd is set to zero in lly
c  the forced terms included are the ones with frequencies
c  indexed from nplin to nplou for planets nplin to nplou
c ********************************************************************
      subroutine forced(g0,s0,pg,ps,pm,pni,pmi,ya,al,xi,et,
     +    nplin,nplou)
      implicit double precision (a-h,o-z)
c  linear secular perturbation theory-- in two dim arrays,
c     first index is planet index, second index is frequency index
c  secular theory constants
      dimension pni(8,8),pg(8),pmi(8,8),ps(8)
c  forced terms
      dimension pxi(8,8),pet(8,8),xi(8),et(8)
c  masses of the planets
      dimension pm(8)
c  yuasa coefficients--warning!! firt index is yuasa index,
c    second index is planet index
c  yuasa coefficients  a
      dimension ya(4,8)
c  for eccentricities
      do 51 k=nplin,nplou
          divg=1.d0/(g0-pg(k))/al
          do 53 j=nplin,nplou
              pxi(j,k)=-pni(j,k)*pm(j)*(ya(4,j))*divg
 53     continue
 51   continue
c  for inclinations
      do 50 k=nplin,nplou
c         if(k.eq.5)goto 50
          divs=1.d0/(s0-ps(k))/al
          do 52 j=nplin,nplou
              pet(j,k)=-pmi(j,k)*pm(j)*(ya(3,j))*divs
 52       continue
 50   continue
c  the forced term at a given frequency is the sum of the contributions
c  from all the planets; that is, mixed terms are included
      do 67 k=nplin,nplou
        xi(k)=0.d0
        et(k)=0.d0
        do 66 j=nplin,nplou
          xi(k)=xi(k)+pxi(j,k)
 66       et(k)=et(k)+pet(j,k)
 67   continue
      return
      end
c **************************************************
      subroutine nlforc(nnl,pninl,xinl,nnlx,ya)
      implicit double precision (a-h,o-z)
c  nonlinear secular perturbation theory for Jupiter, Saturn
      dimension pninl(2,nnl),xinl(nnlx,8)
c  yuasa coefficients--warning!! firt index is yuasa index,
c    second index is planet index
c  yuasa coefficients  a
      dimension ya(4,8)
c  nonlinear forced terms
      do 54 k=1,nnl
           xinl(k,5)=-pninl(1,k)*ya(4,5)
           xinl(k,6)=-pninl(2,k)*ya(4,6)
 54   continue
      return
      end
c
c *******************************************************************
c  propmo version 8 (Apr. 1997)
c  proper modes and proper actions
c  that is removal of the forced terms with frequencies indexed
c  from nplin to nplou
c *******************************************************************
      subroutine propmo(ap,aq,ar,as,al
     +            ,xi,et,pl,pd,nplin,nplou
     +           ,rho)
      implicit double precision (a-h,o-z)
      dimension xi(8),et(8),pl(8),pd(8)
      dimension rho(4)
      sl=dsqrt(al)
c  forced eccentricities and inclinations
      fro1=0.d0
      fro2=0.d0
      fsi1=0.d0
      fsi2=0.d0
      do 1 j=nplin,nplou
          fro1=fro1+xi(j)*cos(pl(j))
          fro2=fro2+xi(j)*sin(pl(j))
          fsi1=fsi1+et(j)*cos(pd(j))
 1        fsi2=fsi2+et(j)*sin(pd(j))
c  proper Poincare' variables
      rho(1)=ap/sl-fro1
      rho(2)=-aq/sl-fro2
      rho(3)=ar/sl-fsi1
      rho(4)=-as/sl-fsi2
      return
      end
c ********************************************************************
c  small routines
c ********************************************************************
c  principal value of an angle
      double precision function princ (a)
      real*8 a,dpig
      data lf/0/
      if(lf.gt.0)goto 1
         dpig=8.d0*datan(1.d0)
         lf=1
  1   princ=dmod(a,dpig)
      if(princ.lt.0.d0)princ=princ+dpig
      return
      end
c **********************************************************
c   given an angle ang and an angle vang with ng revolutions
c   finds a new new ng in the assumption that
c   less than half a revolution has occurred; the new ang
c   replaces vang in the output.
c   in case ang is not between 0 and dpig (eg between
c   -pig and pig, as in the output of datan2), it is reduced
c   to principal value
      subroutine prngup(ang,vang,ng)
      implicit real*8 (a-h,o-z)
      common/trig/dpig
      if(ang.lt.0.d0)ang=ang+dpig
      d=ang-vang
      if(d.gt.dpig*0.5d0)ng=ng-1
      if(d.lt.-dpig*0.5d0)ng=ng+1
      vang=ang
      return
      end

